import axios from 'axios';
import { toast } from 'react-toastify';
import { getLipSyncServer } from '../utils/lipSyncConfig';

// 从配置中获取服务器地址
const getServerUrl = () => {
  return getLipSyncServer();
};

/**
 * 测试对口型服务器连接
 * @param {string} serverUrl - 服务器地址
 * @returns {Promise<{success: boolean, message: string}>} - 测试结果
 */
const testServer = async (serverUrl) => {
  try {
    // 测试服务器
    const response = await axios.get(`${serverUrl}/direct-lip-sync/`, {
      headers: {
        'accept': 'application/json, text/plain, */*',
        'origin': 'https://heygem.fyshark.com',
        'referer': 'https://heygem.fyshark.com/'
      },
      timeout: 5000 // 5秒超时
    });
    
    if (!response.data) {
      return { success: false, message: '服务器响应无效' };
    }
    
    return { success: true, message: '对口型服务器连接正常' };
  } catch (error) {
    console.error('测试对口型服务器失败:', error);
    return { 
      success: false, 
      message: `服务器连接失败: ${error.message}` 
    };
  }
};

/**
 * 提交对口型合成任务
 * @param {string} videoUrl - 视频URL
 * @param {string} audioUrl - 音频URL
 * @returns {Promise<string>} - 返回任务ID
 */
const submitLipSyncTask = async (videoUrl, audioUrl) => {
  try {
    if (!videoUrl || !audioUrl) {
      throw new Error('视频URL和音频URL不能为空');
    }

    const serverUrl = getServerUrl();
    const response = await axios.post(`${serverUrl}/direct-lip-sync/`, {
      video_url: videoUrl,
      audio_url: audioUrl
    }, {
      headers: {
        'accept': 'application/json, text/plain, */*',
        'content-type': 'application/json',
        'origin': 'https://heygem.fyshark.com',
        'referer': 'https://heygem.fyshark.com/'
      }
    });

    if (response.data && response.data.length > 0 && response.data[0].task_id) {
      console.log('对口型任务提交成功:', response.data[0]);
      return response.data[0].task_id;
    } else {
      console.error('对口型任务提交失败:', response.data);
      throw new Error('提交任务失败，未返回任务ID');
    }
  } catch (error) {
    console.error('提交对口型任务错误:', error);
    toast.error(`提交对口型任务失败: ${error.message}`);
    throw error;
  }
};

/**
 * 获取对口型任务信息
 * @param {string} taskId - 任务ID
 * @returns {Promise<Object>} - 返回任务信息
 */
const getLipSyncTaskInfo = async (taskId) => {
  try {
    if (!taskId) {
      throw new Error('任务ID不能为空');
    }

    const serverUrl = getServerUrl();
    const response = await axios.get(`${serverUrl}/direct-lip-sync/?task_id=${taskId}`, {
      headers: {
        'accept': 'application/json, text/plain, */*',
        'origin': 'https://heygem.fyshark.com',
        'referer': 'https://heygem.fyshark.com/'
      }
    });

    if (response.data && response.data.length > 0) {
      console.log('对口型任务信息:', response.data[0]);
      return response.data[0];
    } else {
      console.error('获取任务信息失败:', response.data);
      throw new Error('获取任务信息失败，未返回有效数据');
    }
  } catch (error) {
    console.error('获取对口型任务信息错误:', error);
    toast.error(`获取任务状态失败: ${error.message}`);
    throw error;
  }
};

/**
 * 获取对口型结果视频URL
 * @param {string} resultPath - 结果路径
 * @returns {string} - 完整的结果视频URL
 */
const getLipSyncResultUrl = (resultPath) => {
  if (!resultPath) return null;
  
  const serverUrl = getServerUrl();
  
  // 确保路径以/开头
  const formattedPath = resultPath.startsWith('/') ? resultPath : `/${resultPath}`;
  return `${serverUrl}/download${formattedPath}`;
};

/**
 * 提交对口型任务并定期检查进度直到完成
 * @param {string} videoUrl - 视频URL
 * @param {string} audioUrl - 音频URL
 * @param {function} onProgressUpdate - 进度更新回调函数 (progress, status) => void
 * @param {function} onComplete - 完成回调函数 (resultUrl) => void
 * @param {function} onError - 错误回调函数 (error) => void
 * @returns {Promise<void>}
 */
const processLipSyncTask = async (videoUrl, audioUrl, onProgressUpdate, onComplete, onError) => {
  try {
    // 提交任务
    const taskId = await submitLipSyncTask(videoUrl, audioUrl);
    
    // 定义检查间隔（秒）
    const checkInterval = 5;
    let completed = false;
    
    // 轮询检查任务状态
    const checkStatus = async () => {
      try {
        if (completed) return;
        
        const taskInfo = await getLipSyncTaskInfo(taskId);
        
        // 更新进度
        if (onProgressUpdate) {
          onProgressUpdate(taskInfo.progress, taskInfo.status);
        }
        
        // 检查任务状态
        if (taskInfo.status === 2) { // 已完成
          completed = true;
          const resultUrl = getLipSyncResultUrl(taskInfo.result_url);
          if (onComplete) {
            onComplete(resultUrl, taskInfo);
          }
        } else if (taskInfo.status === 1) { // 进行中
          // 继续轮询
          setTimeout(checkStatus, checkInterval * 1000);
        } else { // 错误状态
          completed = true;
          throw new Error(`任务状态异常: ${taskInfo.status}`);
        }
      } catch (error) {
        if (onError) {
          onError(error);
        }
      }
    };
    
    // 开始轮询
    checkStatus();
    
  } catch (error) {
    console.error('处理对口型任务出错:', error);
    if (onError) {
      onError(error);
    }
  }
};

export default {
  submitLipSyncTask,
  getLipSyncTaskInfo,
  getLipSyncResultUrl,
  processLipSyncTask,
  testServer
}; 